@@ -19,16 +19,18 @@ class Command(ScrapyCommand):
         return "Fetch a URL using the Scrapy downloader"
 
     def long_desc(self):
-        return "Fetch a URL using the Scrapy downloader and print its content " \
-            "to stdout. You may want to use --nolog to disable logging"
+        return (
+            "Fetch a URL using the Scrapy downloader and print its content"
+            " to stdout. You may want to use --nolog to disable logging"
+        )
 
     def add_options(self, parser):
         ScrapyCommand.add_options(self, parser)
-        parser.add_option("--spider", dest="spider", help="use this spider")
-        parser.add_option("--headers", dest="headers", action="store_true",
-                          help="print response HTTP headers instead of body")
-        parser.add_option("--no-redirect", dest="no_redirect", action="store_true",
-                          default=False, help="do not handle HTTP 3xx status codes and print response as-is")
+        parser.add_argument("--spider", dest="spider", help="use this spider")
+        parser.add_argument("--headers", dest="headers", action="store_true",
+                            help="print response HTTP headers instead of body")
+        parser.add_argument("--no-redirect", dest="no_redirect", action="store_true", default=False,
+                            help="do not handle HTTP 3xx status codes and print response as-is")
 
     def _print_headers(self, headers, prefix):
         for key, values in headers.items():
