@@ -8,11 +8,10 @@ def get_engine_status(engine):
     """Return a report of the current engine status"""
     tests = [
         "time()-engine.start_time",
-        "engine.has_capacity()",
         "len(engine.downloader.active)",
         "engine.scraper.is_idle()",
         "engine.spider.name",
-        "engine.spider_is_idle(engine.spider)",
+        "engine.spider_is_idle()",
         "engine.slot.closing",
         "len(engine.slot.inprogress)",
         "len(engine.slot.scheduler.dqs or [])",
@@ -29,7 +28,7 @@ def get_engine_status(engine):
         try:
             checks += [(test, eval(test))]
         except Exception as e:
-            checks += [(test, "%s (exception)" % type(e).__name__)]
+            checks += [(test, f"{type(e).__name__} (exception)")]
 
     return checks
 
@@ -38,7 +37,7 @@ def format_engine_status(engine=None):
     checks = get_engine_status(engine)
     s = "Execution engine status\n\n"
     for test, result in checks:
-        s += "%-47s : %s\n" % (test, result)
+        s += f"{test:<47} : {result}\n"
     s += "\n"
 
     return s
