@@ -5,7 +5,6 @@ library.
 Some of the functions that used to be imported from this module have been moved
 to the w3lib.url module. Always import those from there instead.
 """
-import posixpath
 import re
 from urllib.parse import ParseResult, urldefrag, urlparse, urlunparse
 
@@ -22,7 +21,7 @@ def url_is_from_any_domain(url, domains):
     if not host:
         return False
     domains = [d.lower() for d in domains]
-    return any((host == d) or (host.endswith('.%s' % d)) for d in domains)
+    return any((host == d) or (host.endswith(f'.{d}')) for d in domains)
 
 
 def url_is_from_spider(url, spider):
@@ -31,7 +30,9 @@ def url_is_from_spider(url, spider):
 
 
 def url_has_any_extension(url, extensions):
-    return posixpath.splitext(parse_url(url).path)[1].lower() in extensions
+    """Return True if the url ends with one of the extensions provided"""
+    lowercase_path = parse_url(url).path.lower()
+    return any(lowercase_path.endswith(ext) for ext in extensions)
 
 
 def parse_url(url, encoding=None):
@@ -153,7 +154,7 @@ def strip_url(url, strip_credentials=True, strip_default_port=True, origin_only=
         if (parsed_url.scheme, parsed_url.port) in (('http', 80),
                                                     ('https', 443),
                                                     ('ftp', 21)):
-            netloc = netloc.replace(':{p.port}'.format(p=parsed_url), '')
+            netloc = netloc.replace(f':{parsed_url.port}', '')
     return urlunparse((
         parsed_url.scheme,
         netloc,
